import datetime
import nose.tools as nt

import ckan.lib.helpers as helpers

import ckanext.odsh.helpers as odsh_helpers
from ckanext.odsh.helpers import is_within_last_month
import ckan.model as model


class Test_is_within_last_month(object):
    def test_it_returns_true_for_simple_query(self):
        date = datetime.date(2019, 4, 15)
        date_ref = datetime.date(2019, 4, 29)
        assert is_within_last_month(date, date_ref)

    def test_it_uses_today_if_date_ref_missing(self):
        date = datetime.date.today() - datetime.timedelta(days=20)
        assert is_within_last_month(date)

    def test_it_returns_true_for_dates_in_different_years(self):
        date = datetime.date(2018, 12, 16)
        date_ref = datetime.date(2019, 1, 15)
        assert is_within_last_month(date, date_ref)

    def test_it_returns_false_for_dates_in_different_years(self):
        date = datetime.date(2018, 12, 15)
        date_ref = datetime.date(2019, 1, 15)
        assert is_within_last_month(date, date_ref) == False

    def test_it_returns_true_for_dates_in_differen_months(self):
        date = datetime.date(2018, 6, 16)
        date_ref = datetime.date(2018, 7, 10)
        assert is_within_last_month(date, date_ref)

    def test_it_return_false_for_date_in_different_months(self):
        date = datetime.date(2018, 6, 8)
        date_ref = datetime.date(2018, 7, 10)
        assert is_within_last_month(date, date_ref) == False
